/* -*- c++ -*- */
/*
 * Copyright 2004,2010 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

/*
 * config.h is generated by configure.  It contains the results
 * of probing for features, options etc.  It should be the first
 * file included in your .cc file.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <osmosdr_sink_c.h>
#include <gr_io_signature.h>
#include <gr_audio_sink.h>
#include <gr_complex_to_xxx.h>

/*
 * Create a new instance of osmosdr_sink_c and return
 * a boost shared_ptr.  This is effectively the public constructor.
 */
osmosdr_sink_c_sptr
osmosdr_make_sink_c (const std::string &addr)
{
  return gnuradio::get_initial_sptr(new osmosdr_sink_c (addr));
}

/*
 * Specify constraints on number of input and output streams.
 * This info is used to construct the input and output signatures
 * (2nd & 3rd args to gr_block's constructor).  The input and
 * output signatures are used by the runtime system to
 * check that a valid number and type of inputs and outputs
 * are connected to this block.  In this case, we accept
 * only 1 input and 1 output.
 */
static const int MIN_IN = 1;	// mininum number of input streams
static const int MAX_IN = 1;	// maximum number of input streams
static const int MIN_OUT = 0;	// minimum number of output streams
static const int MAX_OUT = 0;	// maximum number of output streams

/*
 * The private constructor
 */
osmosdr_sink_c::osmosdr_sink_c (const std::string & addr)
  : gr_hier_block2 ("osmosdr_sink_c",
        gr_make_io_signature (MIN_IN, MAX_IN, sizeof (gr_complex)),
        gr_make_io_signature (MIN_OUT, MAX_OUT, sizeof (gr_complex))),
    osmosdr_tx_control(addr)
{
    throw std::runtime_error("FIXME: OsmoSDR sink is not yet implemented");

    /* Audio sink; sample rate is 96kHz by default */
    audio_sink::sptr snk = audio_make_sink(96000, audio_dev_name(), true);

    gr_complex_to_real_sptr real_part = gr_make_complex_to_real(1);
    gr_complex_to_imag_sptr imag_part = gr_make_complex_to_imag(1);

    connect(self(), 0, real_part, 0);
    connect(self(), 0, imag_part, 0);
    connect(imag_part, 0, snk, 0); /* Left is I */
    connect(real_part, 0, snk, 1); /* Right is Q */
}

/*
 * Our virtual destructor.
 */
osmosdr_sink_c::~osmosdr_sink_c ()
{
  // nothing else required in this example
}
